@@ -9,7 +9,7 @@ from typing import TYPE_CHECKING, Any
 try:
     import ujson as json
 except ImportError:
-    import json  # type: ignore
+    import json
 
 import awkward as ak
 from dask.base import tokenize
@@ -181,6 +181,7 @@ def _from_json_files(
     one_obj_per_file: bool = False,
     compression: str | None = "infer",
     meta: ak.Array | None = None,
+    behavior: dict | None = None,
     derive_meta_kwargs: dict[str, Any] | None = None,
     storage_options: dict[str, Any] | None = None,
 ) -> Array:
@@ -217,7 +218,9 @@ def _from_json_files(
             schema=schema,
         )
 
-    return from_map(f, urlpaths, label="from-json", token=token, meta=meta)
+    return from_map(
+        f, urlpaths, label="from-json", token=token, meta=meta, behavior=behavior
+    )
 
 
 def _from_json_bytes(
@@ -227,6 +230,7 @@ def _from_json_bytes(
     blocksize: int | str,
     delimiter: Any,
     meta: ak.Array | None,
+    behavior: dict | None,
     storage_options: dict[str, Any] | None,
 ) -> Array:
     token = tokenize(urlpath, delimiter, blocksize, meta)
@@ -258,7 +262,7 @@ def _from_json_bytes(
     # )
 
     hlg = HighLevelGraph.from_collections(name, dsk, dependencies=deps)
-    return new_array_object(hlg, name, meta=meta, npartitions=n)
+    return new_array_object(hlg, name, meta=meta, behavior=behavior, npartitions=n)
 
 
 def from_json(
@@ -272,13 +276,13 @@ def from_json(
     # initial: int = 1024,
     # resize: float = 1.5,
     highlevel: bool = True,
-    behavior: dict | None = None,
     *,
     blocksize: int | str | None = None,
     delimiter: bytes | None = None,
     one_obj_per_file: bool = False,
     compression: str | None = "infer",
     meta: ak.Array | None = None,
+    behavior: dict | None = None,
     derive_meta_kwargs: dict[str, Any] | None = None,
     storage_options: dict[str, Any] | None = None,
 ) -> Array:
@@ -371,6 +375,7 @@ def from_json(
             one_obj_per_file=one_obj_per_file,
             compression=compression,
             meta=meta,
+            behavior=behavior,
             derive_meta_kwargs=derive_meta_kwargs,
             storage_options=storage_options,
         )
@@ -384,6 +389,7 @@ def from_json(
             delimiter=delimiter,
             blocksize=blocksize,
             meta=meta,
+            behavior=behavior,
             storage_options=storage_options,
         )
 
@@ -450,7 +456,7 @@ def to_json(
     storage_options = storage_options or {}
     fs, _ = url_to_fs(path, **storage_options)
     nparts = array.npartitions
-    write_res = map_partitions(
+    map_res = map_partitions(
         _ToJsonFn(
             fs,
             path,
@@ -464,9 +470,10 @@ def to_json(
         label="to-json-on-block",
         meta=array._meta,
     )
+    map_res.dask.layers[map_res.name].annotations = {"ak_output": True}
     name = f"to-json-{tokenize(array, path)}"
-    dsk = {(name, 0): (lambda *_: None, write_res.__dask_keys__())}
-    graph = HighLevelGraph.from_collections(name, dsk, dependencies=(write_res,))
+    dsk = {(name, 0): (lambda *_: None, map_res.__dask_keys__())}
+    graph = HighLevelGraph.from_collections(name, dsk, dependencies=(map_res,))
     res = new_scalar_object(graph, name=name, meta=None)
     if compute:
         res.compute()
