@@ -811,19 +811,27 @@ def unflatten(array, counts, axis=0, highlevel=True, behavior=None):
 @borrow_docstring(ak.unzip)
 def unzip(
     array: Array, highlevel: bool = True, behavior: dict | None = None
-) -> tuple(Array):
+) -> tuple[Array, ...]:
     if not highlevel:
         raise ValueError("Only highlevel=True is supported")
     fields = ak.fields(array._meta)
     if len(fields) == 0:
         return (array,)
     else:
-        return tuple(array[field] for field in fields)
+        return tuple(array[field] for field in fields)  # type: ignore
 
 
 @borrow_docstring(ak.values_astype)
 def values_astype(array, to, highlevel=True, behavior=None):
-    raise DaskAwkwardNotImplemented("TODO")
+    if not highlevel:
+        raise ValueError("Only highlevel=True is supported")
+    return map_partitions(
+        ak.values_astype,
+        array,
+        to=to,
+        behavior=behavior,
+        label="values-astype",
+    )
 
 
 class _WhereFn:
@@ -882,6 +890,9 @@ class _WithFieldFn:
         self.behavior = behavior
 
     def __call__(self, base: ak.Array, what: ak.Array) -> ak.Array:
+        # TODO: remove backend handling when touch is handled automatically
+        if ak.backend(what) == "typetracer":
+            what.layout._touch_data(recursive=True)
         return ak.with_field(
             base,
             what,
@@ -1085,6 +1096,7 @@ def zip(
             *colls,
             label="zip",
             meta=meta,
+            opt_touch_all=True,
         )
 
     elif isinstance(arrays, (list, tuple)):
